from checkov.graph.terraform.checks_infra.solvers.attribute_solvers.any_attribute_solver import AnyResourceSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_solvers.contains_attribute_solver import ContainsAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_solvers.not_contains_attribute_solver import NotContainsAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_solvers.ending_with_attribute_solver import EndingWithAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_solvers.equals_attribute_solver import EqualsAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_solvers.exists_attribute_solver import ExistsAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_solvers.not_ending_with_attribute_solver import NotEndingWithAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_solvers.not_equals_attribute_solver import NotEqualsAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_solvers.not_exists_attribute_solver import NotExistsAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_solvers.not_starting_with_attribute_solver import NotStartingWithAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_solvers.starting_with_attribute_solver import StartingWithAttributeSolver
from checkov.graph.terraform.checks_infra.solvers.attribute_solvers.within_attribute_solver import WithinAttributeSolver